@@ -1,17 +1,14 @@
 from __future__ import generator_stop
 
 from base64 import urlsafe_b64decode, urlsafe_b64encode
-from typing import TYPE_CHECKING
+from typing import Dict, Hashable, Optional, TypeVar
 from urllib.parse import quote, unquote
 
 from flask import Response, request
 from werkzeug.routing import BaseConverter
 
-if TYPE_CHECKING:
-    from typing import Dict, Hashable, Optional, TypeVar
-
-    T = TypeVar("T")
-    H = TypeVar("H", bound=Hashable)
+T = TypeVar("T")
+H = TypeVar("H", bound=Hashable)
 
 """ examples
 
@@ -27,31 +24,26 @@ def auth():
 
 
 class Base64Converter(BaseConverter):
-    def to_python(self, value):  # decode once as the wsgi app already receives a decoded url
-        # type: (str, ) -> str
+    def to_python(self, value: str) -> str:  # decode once as the wsgi app already receives a decoded url
 
         return urlsafe_b64decode(value.encode("ascii")).decode("utf-8")
 
-    def to_url(self, value):
-        # type: (str, ) -> str
+    def to_url(self, value: str) -> str:
 
         return urlsafe_b64encode(value.encode("utf-8")).decode("ascii")
 
 
 class DoubleQuoteConverter(BaseConverter):
-    def to_python(self, value):  # decode once as the wsgi app already receives a decoded url
-        # type: (str, ) -> str
+    def to_python(self, value: str) -> str:  # decode once as the wsgi app already receives a decoded url
 
         return unquote(value)
 
-    def to_url(self, value):
-        # type: (str, ) -> str
+    def to_url(self, value: str) -> str:
 
         return quote(quote(value, safe=""), safe="")
 
 
-def check_auth(username, password, user_dict):
-    # type: (H, T, Dict[H, T]) -> bool
+def check_auth(username: H, password: T, user_dict: Dict[H, T]) -> bool:
 
     try:
         return user_dict[username] == password
@@ -59,8 +51,7 @@ def check_auth(username, password, user_dict):
         return False
 
 
-def authenticate():
-    # type: () -> Response
+def authenticate() -> Response:
 
     """Sends a 401 response that enables basic auth"""
 
@@ -71,8 +62,7 @@ def authenticate():
     )
 
 
-def do_basic_auth(user_dict):
-    # type: (dict, ) -> Optional[Response]
+def do_basic_auth(user_dict: Dict[H, T]) -> Optional[Response]:
 
     auth = request.authorization
     if not auth or not check_auth(auth.username, auth.password, user_dict):
