@@ -47,12 +47,12 @@ MODE_PAGE_TRANSPORT_GEOMETRY = 0x1E
 MODE_PAGE_DEVICE_CAPABILITIES = 0x1F
 MODE_PAGE_CAPABILITIES = 0x2A
 
-MODE_SENSE_RETURN_ALL = 0x3f
+MODE_SENSE_RETURN_ALL = 0x3F
 
 MODE_SENSE_CURRENT_VALUES = 0x00
 MODE_SENSE_CHANGEABLE_VALUES = 0x40
 MODE_SENSE_DEFAULT_VAULES = 0x80
-MODE_SENSE_SAVED_VALUES = 0xc0
+MODE_SENSE_SAVED_VALUES = 0xC0
 
 # 6-byte commands:
 SCSIOP_TEST_UNIT_READY = 0x00
@@ -250,7 +250,7 @@ SCSIOP_OPERATION32 = 0x7F
 SERVICE_ACTION_OVERWRITE = 0x01
 SERVICE_ACTION_BLOCK_ERASE = 0x02
 SERVICE_ACTION_CRYPTO_ERASE = 0x03
-SERVICE_ACTION_EXIT_FAILURE = 0x1f
+SERVICE_ACTION_EXIT_FAILURE = 0x1F
 
 SERVICE_ACTION_XDWRITE = 0x0004
 SERVICE_ACTION_XPWRITE = 0x0006
@@ -308,57 +308,58 @@ SCSIOP_DENON_READ_SUBCODE = 0xEB
 
 SCSIMESS_ABORT = 0x06
 SCSIMESS_ABORT_WITH_TAG = 0x0D
-SCSIMESS_BUS_DEVICE_RESET = 0X0C
-SCSIMESS_CLEAR_QUEUE = 0X0E
-SCSIMESS_COMMAND_COMPLETE = 0X00
-SCSIMESS_DISCONNECT = 0X04
-SCSIMESS_EXTENDED_MESSAGE = 0X01
-SCSIMESS_IDENTIFY = 0X80
-SCSIMESS_IDENTIFY_WITH_DISCON = 0XC0
-SCSIMESS_IGNORE_WIDE_RESIDUE = 0X23
-SCSIMESS_INITIATE_RECOVERY = 0X0F
-SCSIMESS_INIT_DETECTED_ERROR = 0X05
-SCSIMESS_LINK_CMD_COMP = 0X0A
-SCSIMESS_LINK_CMD_COMP_W_FLAG = 0X0B
-SCSIMESS_MESS_PARITY_ERROR = 0X09
-SCSIMESS_MESSAGE_REJECT = 0X07
-SCSIMESS_NO_OPERATION = 0X08
-SCSIMESS_HEAD_OF_QUEUE_TAG = 0X21
-SCSIMESS_ORDERED_QUEUE_TAG = 0X22
-SCSIMESS_SIMPLE_QUEUE_TAG = 0X20
-SCSIMESS_RELEASE_RECOVERY = 0X10
-SCSIMESS_RESTORE_POINTERS = 0X03
-SCSIMESS_SAVE_DATA_POINTER = 0X02
-SCSIMESS_TERMINATE_IO_PROCESS = 0X11
+SCSIMESS_BUS_DEVICE_RESET = 0x0C
+SCSIMESS_CLEAR_QUEUE = 0x0E
+SCSIMESS_COMMAND_COMPLETE = 0x00
+SCSIMESS_DISCONNECT = 0x04
+SCSIMESS_EXTENDED_MESSAGE = 0x01
+SCSIMESS_IDENTIFY = 0x80
+SCSIMESS_IDENTIFY_WITH_DISCON = 0xC0
+SCSIMESS_IGNORE_WIDE_RESIDUE = 0x23
+SCSIMESS_INITIATE_RECOVERY = 0x0F
+SCSIMESS_INIT_DETECTED_ERROR = 0x05
+SCSIMESS_LINK_CMD_COMP = 0x0A
+SCSIMESS_LINK_CMD_COMP_W_FLAG = 0x0B
+SCSIMESS_MESS_PARITY_ERROR = 0x09
+SCSIMESS_MESSAGE_REJECT = 0x07
+SCSIMESS_NO_OPERATION = 0x08
+SCSIMESS_HEAD_OF_QUEUE_TAG = 0x21
+SCSIMESS_ORDERED_QUEUE_TAG = 0x22
+SCSIMESS_SIMPLE_QUEUE_TAG = 0x20
+SCSIMESS_RELEASE_RECOVERY = 0x10
+SCSIMESS_RESTORE_POINTERS = 0x03
+SCSIMESS_SAVE_DATA_POINTER = 0x02
+SCSIMESS_TERMINATE_IO_PROCESS = 0x11
 
-SCSIMESS_MODIFY_DATA_POINTER = 0X00
-SCSIMESS_SYNCHRONOUS_DATA_REQ = 0X01
-SCSIMESS_WIDE_DATA_REQUEST = 0X03
+SCSIMESS_MODIFY_DATA_POINTER = 0x00
+SCSIMESS_SYNCHRONOUS_DATA_REQ = 0x01
+SCSIMESS_WIDE_DATA_REQUEST = 0x03
 
 SCSIMESS_MODIFY_DATA_LENGTH = 5
 SCSIMESS_SYNCH_DATA_LENGTH = 3
 SCSIMESS_WIDE_DATA_LENGTH = 2
 
+
 class ATA_PASSTHROUGH12(Structure):
-	__slots__ = ()
-	_fields_ = [
-		("OperationCode", UCHAR), # SCSIOP_ATA_PASSTHROUGH12
-		("Reserved1", UCHAR, 1),
-		("Protocol", UCHAR, 4),
-		("MultipleCount", UCHAR, 3),
-		("TLength", UCHAR, 2),
-		("ByteBlock", UCHAR, 1),
-		("TDir", UCHAR, 1),
-		("Reserved2", UCHAR, 1),
-		("CkCond", UCHAR, 1),
-		("Offline", UCHAR, 2),
-		("Features", UCHAR),
-		("SectorCount", UCHAR),
-		("LbaLow", UCHAR),
-		("LbaMid", UCHAR),
-		("LbaHigh", UCHAR),
-		("Device", UCHAR),
-		("Command", UCHAR),
-		("Reserved3", UCHAR),
-		("Control", UCHAR),
-	]
+    __slots__ = ()
+    _fields_ = [
+        ("OperationCode", UCHAR),  # SCSIOP_ATA_PASSTHROUGH12
+        ("Reserved1", UCHAR, 1),
+        ("Protocol", UCHAR, 4),
+        ("MultipleCount", UCHAR, 3),
+        ("TLength", UCHAR, 2),
+        ("ByteBlock", UCHAR, 1),
+        ("TDir", UCHAR, 1),
+        ("Reserved2", UCHAR, 1),
+        ("CkCond", UCHAR, 1),
+        ("Offline", UCHAR, 2),
+        ("Features", UCHAR),
+        ("SectorCount", UCHAR),
+        ("LbaLow", UCHAR),
+        ("LbaMid", UCHAR),
+        ("LbaHigh", UCHAR),
+        ("Device", UCHAR),
+        ("Command", UCHAR),
+        ("Reserved3", UCHAR),
+        ("Control", UCHAR),
+    ]
