@@ -1,25 +1,22 @@
 from __future__ import generator_stop
 
-from typing import TYPE_CHECKING
+from fractions import Fraction
+from typing import Tuple, Union
 
 import numpy as np
-from scipy.ndimage import center_of_mass
 
-from .numpy import bincount_batch, unblock
+from .numpy import bincount_batch, center_of_mass_2d, unblock
 
-if TYPE_CHECKING:
-    from fractions import Fraction
-    from typing import Tuple, Union
-
-    Rational = Union[float, Fraction]
+Rational = Union[float, Fraction]
 
 
-def grayscale(arr):
+def grayscale(arr: np.ndarray) -> np.ndarray:
+    """Converts RGB/BGR images `arr` [batch..., x, y, channel] to grayscale."""
+
     return np.sum(arr, axis=-1) // arr.shape[-1]
 
 
-def histogram_1d(arr, levels):
-    # type: (np.ndarray, int) -> np.ndarray
+def histogram_1d(arr: np.ndarray, levels: int) -> np.ndarray:
 
     """Input shape of `arr`: [batch..., x]. Histogrammed over x and batched over the remaining dimensions.
     Output shape: [batch..., levels]
@@ -28,8 +25,7 @@ def histogram_1d(arr, levels):
     return bincount_batch(arr, -1, levels)
 
 
-def resize_oar(max_width, max_height, dar):
-    # type: (int, int, Rational) -> Tuple[int, int]
+def resize_oar(max_width: int, max_height: int, dar: Rational) -> Tuple[int, int]:
 
     maxdar = max_width / max_height
 
@@ -43,14 +39,12 @@ def resize_oar(max_width, max_height, dar):
     return width, height
 
 
-def resize_maxsize(max_width, max_height, width, height):
-    # type: (int, int, int, int) -> Tuple[int, int]
+def resize_maxsize(max_width: int, max_height: int, width: int, height: int) -> Tuple[int, int]:
 
     return resize_oar(max_width, max_height, width / height)
 
 
-def histogram_2d(arr, levels):
-    # type: (np.ndarray, int) -> np.ndarray
+def histogram_2d(arr: np.ndarray, levels: int) -> np.ndarray:
 
     """Input shape of `arr`: [batch..., y, x]. Histogrammed over x and y and batched over
     the remaining dimensions.
@@ -66,8 +60,7 @@ def histogram_2d(arr, levels):
     return bincount_batch(flattened, -1, levels)
 
 
-def block_histogram_2d(arr, by, bx, levels):
-    # type: (np.ndarray, int, int, int) -> np.ndarray
+def block_histogram_2d(arr: np.ndarray, by: int, bx: int, levels: int) -> np.ndarray:
 
     """Input shape of `arr`: [batch..., y, x]. Histogrammed over blocks of size bx and by
     and batched over the remaining dimensions.
@@ -81,10 +74,9 @@ def block_histogram_2d(arr, by, bx, levels):
     return block_hists.reshape(arr.shape[:-2] + (invy, invx, -1))
 
 
-def image_histogram(arr, levels=256):
-    # type: (np.ndarray, int) -> np.ndarray
+def image_histogram(arr: np.ndarray, levels: int = 256) -> np.ndarray:
 
-    """Input shape of `arr`: [batch..., x, y, channel]. It is summed over channels to create a grayscale
+    """Input shape of RGB/BGR image `arr`: [batch..., x, y, channel]. It is summed over channels to create a grayscale
     image, then histogrammed over x and y and batched over the remaining dimensions.
     Output shape: [batch..., levels]
     """
@@ -96,10 +88,9 @@ def image_histogram(arr, levels=256):
     return histogram_2d(gray, levels)
 
 
-def image_block_histogram(arr, bx, by, levels=256):
-    # type: (np.ndarray, int, int, int) -> np.ndarray
+def image_block_histogram(arr: np.ndarray, bx: int, by: int, levels: int = 256) -> np.ndarray:
 
-    """Input shape of `arr`: [batch..., x, y, channels]. It is summed over channels to create a grayscale
+    """Input shape of RGB/BGR image `arr`: [batch..., x, y, channel]. It is summed over channels to create a grayscale
     image, then histogrammed over x and y and batched over the remaining dimensions.
     Output shape: [batch..., bx, by, levels]
     """
@@ -123,7 +114,7 @@ def center_of_mass_quadrant(img: np.ndarray) -> int:
     if len(img.shape) != 2:
         raise ValueError("Grayscale image expected")
 
-    cm_y, cm_x = center_of_mass(img)
+    cm_y, cm_x = center_of_mass_2d(img)
     d_y, d_x = img.shape
     c_y = (d_y - 1) / 2
     c_x = (d_x - 1) / 2
